package com.faraphel.tasks_valider.database.api.entities

import com.faraphel.tasks_valider.database.api.entities.base.BaseApi
import com.faraphel.tasks_valider.database.dao.base.BaseTaskDao
import com.faraphel.tasks_valider.database.entities.SessionEntity
import com.faraphel.tasks_valider.database.entities.SubjectEntity
import com.faraphel.tasks_valider.utils.getBody
import com.google.gson.Gson
import fi.iki.elonen.NanoHTTPD


class SubjectApi(
    private val dao: BaseTaskDao<SubjectEntity>,
    private val session: SessionEntity
) : BaseApi {
    companion object {
        private val parser = Gson()  ///< The JSON parser
    }

    // Requests

    override fun head(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        // get the content of the request
        val data = httpSession.getBody()
        // parse the object
        val obj = parser.fromJson(data, SubjectEntity::class.java)
        // check if the object is in the object accessible from the session
        val exists = this.dao.getAllBySession(session.id).contains(obj)

        return NanoHTTPD.newFixedLengthResponse(
            if (exists) NanoHTTPD.Response.Status.OK else NanoHTTPD.Response.Status.NOT_FOUND,
            "text/plain",
            if (exists) "Exists" else "Not found"
        )
    }

    override fun get(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        return NanoHTTPD.newFixedLengthResponse(
            NanoHTTPD.Response.Status.OK,
            "application/json",
            parser.toJson(this.dao.getAllBySession(session.id))
        )
    }

    override fun post(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        // get the content of the request
        val data = httpSession.getBody()
        // parse the object
        val obj = parser.fromJson(data, SubjectEntity::class.java)
        val id = this.dao.insert(obj)

        return NanoHTTPD.newFixedLengthResponse(
            NanoHTTPD.Response.Status.CREATED,
            "text/plain",
            id.toString()
        )
    }

    override fun delete(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        // get the content of the request
        val data = httpSession.getBody()
        // parse the object
        val obj = parser.fromJson(data, SubjectEntity::class.java)
        val count = this.dao.delete(obj)

        return NanoHTTPD.newFixedLengthResponse(
            if (count > 0) NanoHTTPD.Response.Status.OK else NanoHTTPD.Response.Status.NOT_FOUND,
            "text/plain",
            count.toString()
        )
    }
}