package com.faraphel.tasks_valider.ui.screen.task

import android.app.Activity
import android.os.Build
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.compose.foundation.layout.*
import androidx.compose.material3.Button
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.database.entities.PersonEntity
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken


/**
 * This screen represent a session
 * @param activity the android activity
 * @param client an HTTP client that can communicate with the server
 */
@RequiresApi(Build.VERSION_CODES.O)
@Composable
fun TaskSessionScreen(
    activity: Activity,
    client: TaskClient,
    user: PersonEntity,
) {
    val students = remember { mutableStateOf<List<PersonEntity>?>(null) }
    val selectedStudent = remember { mutableStateOf<PersonEntity?>(null) }

    // if the groups are not yet defined, refresh the list
    if (students.value == null)
        return Thread { refreshStudents(activity, client, students) }.start()

    if (selectedStudent.value != null)
        return TaskStudentScreen(
            activity,
            client,
            user,
            selectedStudent.value!!
        )

    Column(
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        // title
        Text(
            text = "Session",
            fontSize = 32.sp
        )

        // separator
        Spacer(modifier = Modifier.height(24.dp))

        // if the groups have already been defined, display them
        for (student in students.value!!) {
            Button(
                modifier = Modifier.fillMaxWidth().padding(vertical = 2.dp, horizontal = 16.dp),
                onClick = { selectedStudent.value = student },
            ) {
                Text(text = student.fullName())
            }
        }
    }
}


// TODO(Faraphel): template this function ?
fun refreshStudents(
    activity: Activity,
    client: TaskClient,
    students: MutableState<List<PersonEntity>?>
) {
    // TODO(Faraphel): global variable ?
    val jsonParser = Gson()

    // try to obtain the list of groups
    val response = client.get("entities/" + PersonEntity.TABLE_NAME)

    // in case of error, notify it
    if (!response.isSuccessful)
        return activity.runOnUiThread { Toast.makeText(activity, response.message, Toast.LENGTH_LONG).show() }

    // parse the list of groups
    students.value = jsonParser.fromJson(
        response.body.string(),
        object : TypeToken<List<PersonEntity>>(){}.type
    )
}
